@@ -6,7 +6,7 @@ use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '2.18.0';
+our $VERSION = '2.19.0';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
@@ -783,14 +783,24 @@ sub metadata {
     if ( $req->params('full') and $req->params('full') !~ NO ) {
         my $c = $self->getConfKey( $req, 'cfgNum' );
         return $self->sendError( $req, undef, 400 ) if ( $req->error );
-        if ( $self->can('userId') ) {
-            $self->userLogger->notice( 'User '
-                  . $self->userId($req)
-                  . ' ask for full configuration '
-                  . $c );
+        if ( my $userid = $self->_userId($req) ) {
+            $self->auditLog(
+                $req,
+                message => (
+                    'User ' . $userid . ' ask for full configuration ' . $c
+                ),
+                code   => "CONFIG_DOWNLOADED",
+                user   => $userid,
+                cfgNum => $c
+            );
         }
         else {
-            $self->logger->info("REST request to get full configuration $c");
+            $self->auditLog(
+                $req,
+                message => "REST request to get full configuration $c",
+                code    => "CONFIG_DOWNLOADED",
+                cfgNum  => $c
+            );
         }
         return $self->sendJSONresponse(
             $req,
@@ -801,6 +811,36 @@ sub metadata {
             ],
         );
     }
+    elsif ( $req->params('oidcMetadata') ) {
+        my $c = $self->getConfKey( $req, 'cfgNum' );
+        return $self->sendError( $req, undef, 400 ) if ( $req->error );
+        if ( $self->can('userId') ) {
+            $self->userLogger->notice( 'User '
+                  . $self->userId($req)
+                  . ' ask for OIDC metadata '
+                  . $c );
+        }
+        else {
+            $self->logger->info("REST request to get OIDC metadata $c");
+        }
+        require Lemonldap::NG::Common::OpenIDConnect::Metadata;
+        my $path = $self->currentConf->{issuerDBOpenIDConnectPath};
+        $path =~ s#^.*?(\w+).*?$#$1/#;
+        return $self->sendJSONresponse(
+            $req,
+            Lemonldap::NG::Common::OpenIDConnect::Metadata->metadataDoc(
+                $self->currentConf->{oidcServiceMetaDataIssuer}
+                  || $self->p->portal,
+                $self->currentConf,
+                $path,
+            ),
+            pretty  => 1,
+            headers => [
+                'content-Type'        => 'application/json',
+                'Content-Disposition' => "Attachment; filename=openid-configuration.json"
+            ],
+        );
+    }
     else {
         my $res = {};
         $res->{cfgNum} = $self->getConfKey( $req, 'cfgNum' );
@@ -819,16 +859,28 @@ sub metadata {
         if ( defined $ind and $ind < $#a ) {
             $res->{next} = $a[ $ind + 1 ];
         }
-        if ( $self->can('userId') ) {
-            $self->userLogger->info( 'User '
-                  . $self->userId($req)
-                  . ' ask for configuration metadata ('
-                  . $res->{cfgNum}
-                  . ')' );
+        if ( my $userid = $self->_userId($req) ) {
+            $self->auditLog(
+                $req,
+                code    => "CONFIG_GET_METADATA",
+                user    => $userid,
+                cfgNum  => $res->{cfgNum},
+                message => (
+                        'User '
+                      . $userid
+                      . ' ask for configuration metadata ('
+                      . $res->{cfgNum} . ')'
+                )
+            );
         }
         else {
-            $self->logger->info(
-                "REST request to get configuration metadata ($res->{cfgNum})");
+            $self->auditLog(
+                $req,
+                code    => "CONFIG_GET_METADATA",
+                cfgNum  => $res->{cfgNum},
+                message =>
+                  "REST request to get configuration metadata ($res->{cfgNum})",
+            );
         }
         return $self->sendJSONresponse( $req, $res );
     }
@@ -849,12 +901,22 @@ sub getKey {
     unless ($key) {
         return $self->metadata($req);
     }
-    if ( $self->can('userId') ) {
-        $self->userLogger->info(
-            'User ' . $self->userId($req) . " asks for key $key" );
+    if ( my $userid = $self->_userId($req) ) {
+        $self->auditLog(
+            $req,
+            code    => "CONFIG_GET_KEY",
+            message => ( 'User ' . $userid . " asks for key $key" ),
+            user    => $userid,
+            key     => $key,
+        );
     }
     else {
-        $self->logger->info("REST request to get configuration key $key");
+        $self->auditLog(
+            $req,
+            code    => "CONFIG_GET_KEY",
+            message => ("REST request to get configuration key $key"),
+            key     => $key,
+        );
     }
     my $value = $self->getConfKey( $req, $key );
     return $self->sendError( $req, undef, 400 ) if ( $req->error );
@@ -906,4 +968,12 @@ sub getKey {
     # TODO authParam key
 }
 
+sub _userId {
+    my ( $self, $req ) = @_;
+    if ( $self->can('p') and $self->p->can('userId') ) {
+        return $self->p->userId($req);
+    }
+    return;
+}
+
 1;
